@@ -22,7 +22,7 @@
 #include "config.h"
 #endif
 
-#include <assert.h>
+#include <winpr/assert.h>
 
 #include <freerdp/freerdp.h>
 
@@ -31,8 +31,6 @@
 
 #include "utils.h"
 
-#define WINPR_ASSERT(x) assert(x)
-
 BOOL utils_abort_connect(rdpContext* context)
 {
 	WINPR_ASSERT(context);
